<?php
namespace Teknisa\EST\Controllers;

use Doctrine\ORM\EntityManager;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;

use Zeedhi\Framework\DataSource\DataSet;

use Teknisa\EST\Services\Est53000ParametrosDeEstoqueService;

class Est53000ParametrosDeEstoqueController extends \Teknisa\EST\Controllers\TeknisaCrud{
    #default services
    protected $environment;
    protected $service;
    protected $crudService;
    protected $dataSourceManager;

    /* **************************************************
     * Construtor do Controller.
     *
     * @param Environment                        $environment
     * @param Est53000ParametrosDeEstoqueService $service
     */
    public function __construct($crudService, $parameterBag,$dataSourceManager,$environment, Est53000ParametrosDeEstoqueService $service) {
        #initializes default services
        parent::__construct($crudService, $parameterBag);
		$this->crudService = $crudService;
        $this->environment   = $environment;
        $this->service       = $service;
        $this->dataSourceManager = $dataSourceManager;
    }

    public function deleteAllDays(DTO\Request\Filter $request, DTO\Response $response) {
        #gets filter parameters
        $filters = $request->getFilterCriteria()->getConditions();

        #get values
        $CDFILIAL = $filters[0]['value'];
        $NRORG    = $this->environment->getNrOrgTrab();

        try {
            #tries to remove every persisted day
            $this->service->deleteAllDays($CDFILIAL, $NRORG);
        } catch(\Exception $e) {
            #catches error and sets message as needed
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function getDataForGeral(DTO\Request\Filter $request, DTO\Response $response) {
        #default values
        $data = null;

        #get values
        $NRORG = $this->environment->getNrOrgTrab();

        try {
            #tries to get generic parametrization data
            $data = $this->service->getDataForGeral($NRORG);
            #returns data
            $response->addDataSet(new DataSet("paramestoque_geral", $data));
        } catch(\Exception $e) {
            #catches error and sets message as needed
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function getDataForProdutoField(DTO\Request\Filter $request, DTO\Response $response) {
        #default values
        $data = null;

        #get values
        $NRORG = $this->environment->getNrOrgTrab();

        try {
            #tries to get branch parametrization data
            $data = $this->service->getDataForProdutoField($NRORG);

            #returns data
            $response->addDataSet(new DataSet("paramestoque_produto_field", $data));
        } catch(\Exception $e) {
            #catches error and sets message as needed
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function validateProductInsertion(DTO\Request\Filter $request, DTO\Response $response) {
        #get values
        $NRORG = $this->environment->getNrOrgTrab();

        #gets filter parameters
        $filters = $request->getFilterCriteria()->getConditions();

        #gets data
        $CDFILIAL     = $filters[0]['value'];
        $CDALMOXARIFE = $filters[1]['value'];
        $CDLOCALESTOQ = $filters[2]['value'];
        $CDPRODUTO    = $filters[3]['value'];
        $CDARVPROD    = $filters[4]['value'];
        $IDCURVAABC   = $filters[5]['value'];

        try {
            #validates product insertion
            $valid = $this->service->validateProductInsertion($CDPRODUTO, $CDARVPROD, $IDCURVAABC, $CDALMOXARIFE, $CDLOCALESTOQ, $CDFILIAL, $NRORG);

            #returns validation
            $response->addDataSet(new DataSet("paramestoque_produto_validation", array($valid)));
        } catch(\Exception $e) {
            #catches error and sets message as needed
            $response->setError(new Error("Erro ao salvar os dados. ".$e->getMessage(), 500));
        }
    }

    public function saveProduct(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        $CDFILIAL     = $row['CDFILIAL'];
        $CDALMOXARIFE = $row['CDALMOXARIFE'];
        $CDLOCALESTOQ = $row['CDLOCALESTOQ'];
        $IDCURVAABC   = $row['IDCURVAABC'];
        $CDPRODUTO    = $row['CDPRODUTO'];
        $NRORG        = $this->environment->getNrOrgTrab();
        try {

            if($row['__is_new']){
               $this->service->saveProduct($CDFILIAL, $CDALMOXARIFE, $CDLOCALESTOQ, $IDCURVAABC, $CDPRODUTO, $NRORG);
            }else{
                $IDCURVAABC_OLD   = $row['IDCURVAABC_OLD'];
                $this->service->updateProduct($CDFILIAL, $CDALMOXARIFE, $CDLOCALESTOQ, $IDCURVAABC, $CDPRODUTO, $NRORG, $IDCURVAABC_OLD);
            }

        } catch(\Exception $e) {
            #catches error and sets message as needed
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function deleteProdutosCurvaAbc(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $row['NRORG'] = $this->environment->getNrOrgTrab();
            $this->service->deleteProdutosCurvaAbc($row);
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function saveGeral(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            if($row['WIDGETNAME'] === 'inventario') $this->saveOperatorsGroup($row);
            $this->service->saveGeral($row);
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function saveUnidade(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $this->service->validateStockControll($row);
            $this->service->saveUnidade($row);
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function saveParametrosCurvaAbc(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $this->service->saveParametrosCurvaAbc($row);
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function getDataForProdutoWidget(DTO\Request\Filter $request, DTO\Response $response) {
        #default values
        $data = null;

        #gets filter parameters
        $filters = $request->getFilterCriteria()->getConditions();

        #get values
        $CDALMOXARIFE = $filters[0]['value'];
        $CDLOCALESTOQ = $filters[1]['value'];
        $CDFILIAL     = $filters[2]['value'];
        $NRORG        = $this->environment->getNrOrgTrab();

        try {
            #tries to get branch parametrization data
            $data = $this->service->getDataForProdutoWidget($CDALMOXARIFE, $CDLOCALESTOQ, $CDFILIAL, $NRORG);

            #returns data
            $response->addDataSet(new DataSet("paramestoque_produto_widget", $data));
        } catch(\Exception $e) {
            #catches error and sets message as needed
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function getDataForUnidade(DTO\Request\Filter $request, DTO\Response $response) {
        #default values
        $data = null;

        #gets filter parameters
        $filters = $request->getFilterCriteria()->getConditions();

        #get values
        $CDFILIAL = $filters[0]['value'];
        $NRORG    = $this->environment->getNrOrgTrab();

        try {
            #tries to get branch parametrization data
            $data = $this->service->getDataForUnidade($CDFILIAL, $NRORG);

            #returns data
            $response->addDataSet(new DataSet("paramestoque_unidade", $data));
        } catch(\Exception $e) {
            #catches error and sets message as needed
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function saveOperatorsGroup($row) {
        $arrayGroupCode = [];
            
        $groupArrayToSave = [];
        
        forEach($row as $field=>$value){
            switch($field){
                case'CDGRUPGEST':
                    $this->groupArrayBuilder($row['CDGRUPGEST'],'GEST',$groupArrayToSave,$arrayGroupCode);
                    break;
                case'CDGRUPGGER':
                    $this->groupArrayBuilder($row['CDGRUPGGER'],'GGER',$groupArrayToSave,$arrayGroupCode);
                    break;
                case'CDGRUPGAUD':
                    $this->groupArrayBuilder($row['CDGRUPGAUD'],'GAUD',$groupArrayToSave,$arrayGroupCode);
                    break;
                default:
                    break;
            }
        }
        
        if($this->array_has_duplicates($arrayGroupCode)){
            throw new \Exception("Operação bloqueada! Não é permitido a parametrização de um grupo de operadores já parametrizado para outro perfil do estoque cego.",500);
        }
        
		$this->service->clearExistingGroups();
		
        if(!empty($groupArrayToSave)){
            $newDatasetToSave = new DataSet('autgrupop', $groupArrayToSave);
    		$newRequestToSave = new DTO\Request\DataSet($newDatasetToSave, '', '', '');
    		$groupsToSave = $this->crudService->getRowsToSave($newRequestToSave, "autgrupop");
    		$dataSetToSave = new DataSet("autgrupop", $groupsToSave);
            $this->dataSourceManager->persist($dataSetToSave);
        }
    }
    
    public function groupArrayBuilder($CDGRUPOPER,$IDAUTGRUPOP,&$ArrayToSave,&$arrayGroupCode){
        if(!empty($CDGRUPOPER)){
            $array = array(
                "CDGRUPOPER"  => $CDGRUPOPER,
                "IDAUTGRUPOP" => $IDAUTGRUPOP,
                "NRAUTGRUPOP" => '000',
                "__is_new"    => true
            );
            array_push($ArrayToSave, $array);
            array_push($arrayGroupCode,$CDGRUPOPER);
        }
    }
    
    public function array_has_duplicates($array) {
       return count($array) !== count(array_unique($array));
    }
    
}