var Est53000_Parametros_de_Estoque_Controller = function(est53000_parametros_de_estoque_service, ZhFilterPreferences, General, ScreenService, templateManager, ZHPromise, EstUtil) {
    var self = this;
    var once = 0;

    /* **************************************************
     * Função que cria um backup dos dados da aba de Dias
     * do Inventário Rotativo, marcando-os como "changed"
     * para que sejam re-inseridos caso necessária a
     * restauração do backup.
     *
     * @param Widget widget
     *
     * @return Promise
     */
    self.backupRealizationDaysData = function(widget) {
        //reloads widget to get all data
        return widget.reload().then(function() {
            //checks whether there should be any data in widget
            if (!widget.enabled || widget.dataSource.data.length < 1)
                //creates invalid backup in case of invalid parametrization
                widget.backupDataset = false;
            else {
                //backups data
                widget.backupDataset = widget.dataSource.data;

                //sets every backup row as changed in order to correctly apply backup later
                widget.backupDataset.forEach(function(row) {
                    row.changed = true;
                });
            }

            //resolves promise with created backup
            return widget.backupDataset;
        });
    };

    /* **************************************************
     * Função que cancela a edição de dados na aba de
     * inventário rotativo, efetuando o controle de seu
     * fluxo apropriadamente e restaurando o backup dos
     * dias previamente parametrizados.
     *
     * @param Widget widget
     *
     * @return Promise
     */
    self.cancelRotativeInventoryEditing = function(widget) {
        //gets widgets
        var tabbedWdgt = widget.getParent();
        var formWidget = tabbedWdgt.widgets[0];
        var gridWidget = tabbedWdgt.widgets[1];

        //returns to form widget to avoid validation errors
        if (!formWidget.isActive())
            formWidget.activate();

        //cancels editing and returns widget to normal operation
        General.widgetCancelForm(formWidget);
        General.changeWidgetAcessibility(tabbedWdgt.getParent().widgets.concat(tabbedWdgt.getParent().getParent().widgets), true);

        //returns backup of grid widget
        return self.restoreRealizationDaysBackup(gridWidget).then(function() {
            //performs validations
            self.checkWidget(formWidget);

            //checks grid widget actions
            self.checkRealizationDaysActionsVisibility(gridWidget);
        });
    };

    /* **************************************************
     * Procedimento que muda a acessibilidade das widgets
     * recebidas e de suas actions.
     *
     * @param Array widgets
     * @param boolean enabled
     */
    self.changeSubWidgetAcessibility = function(widgets, enabled) {
        //iterates over recieved widgets
        widgets.forEach(function(widget) {
            //changes widget acessibility
            widget.enabled = enabled;
            //changes action acessibility
            widget.actions.forEach(function(action) {
                action.readOnly = !enabled;
            });
        });

        //updates template to exhibit changes
        templateManager.updateTemplate();
    };

    /* **************************************************
     * Procedimento que checa a validade de algum valor
     * percentual. Possui mensagens padrão para campos
     * específicos.
     *
     * @param Field field
     */
    self.checkAdmittedPercentage = function(field) {
        var value = field.widget.currentRow[field.name];

        //parametrized error messages
        var msgs = {
            VRPERCINVROT: "Percentual para inventário rotativo deve estar entre 0 e 100%.",
            VRPETOLEENTR: "Percentual para tolerância admitida deve estar entre 0 e 100%.",
            VRPETOLEINVROT: "Percentual para tolerância admitida deve estar entre 0 e 100%.",
            DEFAULT: "Valor percentual deve estar entre 0 e 100%."
        };

        //in case of invalid percentage
        if (value !== null && (0 > value || value > 100)) {
            //cleans invalid data
            field.widget.currentRow[field.name] = null;

            //picks error message
            var msg = (msgs[field.name]) ? msgs[field.name] : msgs.DEFAULT;

            ScreenService.showMessage(msg);
        }
    };

    /* **************************************************
     * Procedimento que checa a parametrização de
     * utilização de controle de lote e modifica a
     * disponibilidade dos campos dos grupos "Descrição
     * de Número de Lote" e "Parâmetros para Controle de
     * Lote" de acordo.
     *
     * @param Widget widget
     */
    self.checkAllotment = function(widget) {
        var check = widget.currentRow.IDUTILLOTE != 'S';
        var subFieldNames = ["DSNUMELOTE", "DSSEQULOTE", "DSORDENALOTE", "IDENTRSEQLOTE", "IDENTRAUTLOTE", "IDENTRUNILOTE", "CDGRUPPRCTRLTEM"];

        //iterates over every dependent field
        subFieldNames.forEach(function(fieldName) {
            var field = widget.getField(fieldName);

            //changes field avaliability according to parametrization
            field.readOnly = check;

            //in case of negative parametrization, cleans field data
            if (check) {
                if (field.fieldGroup == 'DNL' || field.fieldGroup == 'ADP') //nullifies text fields
                    widget.currentRow[fieldName] = null;
                else { //set negative parametrization for option fields
                    if (field.name === '')
                        widget.currentRow[field.valueField] = 'N';

                }

            }
        });
        self.checkAllotmentNumberDescription(widget);
    };

    /* **************************************************
     * Procedimento que checa a parametrização de número
     * de lote e modifica a disponibilidade do campo de
     * sequencial de lote de acordo.
     *
     * @param Widget widget
     */
    self.checkAllotmentNumberDescription = function(widget) {
        var subField = null;
        //checks parameter existence
        if (widget.currentRow.DSNUMELOTE) {
            //searches for specific substring that makes DSSEQULOTE field required
            var check = widget.currentRow.DSNUMELOTE.search("\\[SEQ") != -1;
            subField = widget.getField('DSSEQULOTE');

            //verifies if current condition is diferent from what was computed and executes changes as necessary
            if (check ^ subField.validations.required)
                this.switchFieldRequirement(subField);

        }
        else {
            subField = widget.getField('DSSEQULOTE');
            delete(subField.validations.required);
        }
    };

    /* **************************************************
     * Função que troca a condição "required" de um field
     * e retorna um boolean que indica se o field passou
     * ou não a sê-lo.
     *
     * @param Field field
     *
     * @return boolean
     */
    self.switchFieldRequirement = function(field) {
        if (field.validations) {
            if (field.validations.required) {
                delete field.validations.required;
                return false;
            }
            else {
                field.validations.required = true;
                return true;
            }
        }
        return false;
    };

    /* **************************************************
     * Procedimento que checa a validade do campo de
     * ajuste/implantação automática e altera a
     * disponibilidade de seu campo de seleção de acordo.
     *
     * @param Widget widget
     */
    self.checkAutomaticInventoryDeploymentGeneration = function(widget) {
        var check = widget.currentRow.IDAJUSIMPLAU != 'S';
        var subField = widget.getField('IDOBRIGAAJUINVE');

        //changes field avaliability according to parametrization
        subField.readOnly = check;

        if (check) //in case of field unavailability, sets negative parametrization
            widget.currentRow.IDOBRIGAAJUINVE = "F";
    };

    /* **************************************************
     * Procedimento que valida a parametrização de
     * inventário rotativo à partir do campo "Realiza
     * Inventário por Estoque Cego", habilitando e
     * desabilitando o acesso à metade dos campos, de
     * acordo com o valor observado.
     *
     * @param Widget widget
     */
    self.BlindInventoryParametrization = function(widget) {
        let checkParamsInvRot = (
            widget.currentRow.IDINVROTEST == 'S' || widget.currentRow.IDINVEROTREQ == 'S' ||
            widget.currentRow.IDCALCRECBRUTA == 'S' || widget.currentRow.IDINVROTDIA == 'S' ||
            widget.currentRow.IDAJUSIMPLAU == 'S' || widget.currentRow.IDPERMVISPOSEST == 'S'
        );

        var field = widget.getField('IDINVESTCEGO');
        if (checkParamsInvRot) {
            field.readOnly = true;
            widget.currentRow.IDINVESTCEGO = 'N';
        }
        else {
            field.readOnly = false;
        }

    };

    self.checkBlindInventoryRealization = function(widget) {
        var check = widget.currentRow.IDINVESTCEGO == 'S';

        //default empty values
        var subFieldNames = [
            {
                "IDINVROTEST": "N",
                "IDINVEROTREQ": "N",
                "VRPETOLEINVROT": null,
                "VRPERCINVROT": null,
                "IDINVROTFMES": "S",
                "NRDIAFECHMES": null,
                "IDINVROTDIA": "N",
                "IDAJUSIMPLAU": "N",
                "IDPERMVISPOSEST": "N"
            },
            {
                "NMSERVICO": null,
                "NMTIPORETI": null,
                "IDBLOQFECINVPE": "N",
                "IDOBREALACEDIV": "N"
            }
        ];

        //iterates over first group of dependent fields
        Object.keys(subFieldNames[0]).forEach(function(fieldName) {
            var field = widget.getField(fieldName);

            //changes field avaliability according to parametrization
            field.readOnly = check;

            //in case of negative parametrization, cleans field data
            if (check)
                widget.currentRow[fieldName] = subFieldNames[0][fieldName];
        });

        //inverts the value of the check variable to obtain the opposite effect in the blind inventory fields
        check = !check;

        //iterates over second group of dependent fields
        Object.keys(subFieldNames[1]).forEach(function(fieldName) {
            var field = widget.getField(fieldName);

            //changes field avaliability according to parametrization
            field.readOnly = check;

            //in case of negative parametrization, cleans field data
            if (check)
                widget.currentRow[fieldName] = subFieldNames[1][fieldName];
        });

        //checks rotative inventory days widget
        self.checkRealizationDaysWidget(widget);

        //unable field Tolerância Admitida (%)
        widget.getField('VRPETOLEINVROT').readOnly = true;
    };

    /* **************************************************
     * Procedimento que controla a troca de unidade da
     * aba de mesmo nome. Checa a validade do contrato da
     * unidade escolhida para a tab, mostrando uma
     * mensagem de aviso caso o mesmo esteja rescindido.
     *
     * Retorna o foco para a primeira aba filha e
     * requisita os dados da unidade selecionada.
     *
     * @param Widget widget
     */
    self.checkBranch = function(widget) {
        var promiseTipoCusto = ZHPromise.defer();
        widget.promise = promiseTipoCusto;

        //gets current active subwidget
        var activeWidget = widget.container.getActiveWidget();
        //gets first subwidget
        var firstWidget = widget.widgets[0];
        //default value
        var promise = false;

        //checks if branch code value is valid
        var check = widget.currentRow.CDFILIAL !== "" &&
            widget.currentRow.CDFILIAL !== null &&
            widget.currentRow.CDFILIAL !== undefined;

        if (!check && once == 0) {
            this.getFilialLogadaOperador().then(function(success) {
                let filial = success.dataset.filial[0];
                if (filial.IDCNTRRESC == 'S') {
                    widget.currentRow.CDFILIAL = '';
                    widget.currentRow.NMFILIAL = '';
                    ScreenService.showMessage('Filial com o contrato rescindido.');
                }
                else {
                    widget.currentRow.CDFILIAL = filial.CDFILIAL;
                    widget.currentRow.NMFILIAL = filial.NMFILIAL;

                    self.replicateDataForChildren(widget);


                    once = 1;
                    check = true;

                    //checks if "Inventário Rotativo" houses the current active widget
                    if (activeWidget.name == 'paramsrotativo' || activeWidget.name == 'diasrotativo') {
                        //checks if editing is currently in progress and cancels it, receiving a promise
                        if (activeWidget.getParent().widgets[0].mode == 'edit')
                            promise = self.cancelRotativeInventoryEditing(activeWidget);
                    }
                    else if (activeWidget.mode == 'edit') {
                        //cancels other generic editing cases
                        General.widgetCancelForm(activeWidget);

                        //restore acessibility for subwidgets
                        General.changeWidgetAcessibility(widget.getParent().widgets, true);
                    }

                    //checks if a promise was received and creates one otherwise
                    if (!promise) {
                        promise = ZHPromise.defer();

                        //resolved by default
                        promise.resolve();
                    }

                    //waits for other processes to be done
                    promise.then(function() {
                        //changes widget acessibility accordingly
                        self.changeSubWidgetAcessibility(widget.widgets, check);

                        //activates first widget if necessary
                        if (activeWidget != firstWidget)
                            firstWidget.activate();

                        //gets data for "Unidade" tab and its subwidgets
                        self.getDataForUnidadeWidget(widget);
                    });
                }

            }.bind(this));
        }
        else {
            if (widget.currentRow.IDCNTRRESC == 'S') {
                widget.currentRow.CDFILIAL = '';
                widget.currentRow.NMFILIAL = '';
                ScreenService.showMessage('Filial com o contrato rescindido.');
            }
            else {
                //checks if "Inventário Rotativo" houses the current active widget
                if (activeWidget.name == 'paramsrotativo' || activeWidget.name == 'diasrotativo') {
                    //checks if editing is currently in progress and cancels it, receiving a promise
                    if (activeWidget.getParent().widgets[0].mode == 'edit')
                        promise = self.cancelRotativeInventoryEditing(activeWidget);
                }
                else if (activeWidget.mode == 'edit') {
                    //cancels other generic editing cases
                    General.widgetCancelForm(activeWidget);

                    //restore acessibility for subwidgets
                    General.changeWidgetAcessibility(widget.getParent().widgets, true);
                }

                //checks if a promise was received and creates one otherwise
                if (!promise) {
                    promise = ZHPromise.defer();

                    //resolved by default
                    promise.resolve();
                }

                //waits for other processes to be done
                promise.then(function() {
                    //changes widget acessibility accordingly
                    self.changeSubWidgetAcessibility(widget.widgets, check);

                    //activates first widget if necessary
                    if (activeWidget != firstWidget)
                        firstWidget.activate();

                    //gets data for "Unidade" tab and its subwidgets
                    self.getDataForUnidadeWidget(widget);

                });
            }
        }

    };

    /* **************************************************
     * Procedimento que atualiza o filtro do campo
     * "Serviço para Baixa de Quantidades Divergentes"
     * para corresponder à unidade selecionada da aba
     * "Unidade".
     *
     * @param Widget widget
     */
    self.checkBranchForServiceForStockReductionField = function(widget) {
        var field = widget.getField("NMSERVICO");

        //builds filter
        var filter = {};
        filter.name = "CDFILIAL";
        filter.operator = "=";
        filter.value = widget.getParent().getParent().currentRow.CDFILIAL;

        //adds filter
        field.dataSourceFilter = [];
        field.dataSourceFilter.push(filter);

        //reloads data
        field.reload();
    };

    /* **************************************************
     * Procedimento que checa a parametrização de
     * utilização de controle por almoxarifado e modifica
     * a disponibilidade do campo de utilização de
     * controle de local de estoque de acordo.
     *
     * @param Widget widget
     */
    self.checkControlUsage = function(widget) {
        var check = widget.currentRow.IDUTILALMOX != 'S';
        var subField = widget.getField('IDUTILLCESTQ');

        //changes field avaliability according to parametrization
        subField.readOnly = check;

        if (check) //in case of field unavailability, sets negative parametrization
            widget.currentRow.IDUTILLCESTQ = 'N';
    };

    self.setOldControlUsageOnRow = function(widget) {
        let arrFieldsToStore = ['IDUTILALMOX', 'IDUTILLCESTQ', 'IDUTILLOTE'];
        arrFieldsToStore.forEach(field => widget.currentRow['old_' + field] = widget.currentRow[field]);
    };

    self.editCurveABC = function(widget) {
        let wCurrentRow = widget.currentRow;
        let wParentRow = widget.getParent().currentRow;
        General.widgetEdit(widget);
        //restaura valores da curva abc removidos pelo edit.
        setTimeout(call => {
            wCurrentRow.VRPECURVAA = wParentRow.VRPECURVAA;
            wCurrentRow.VRPECURVAB = wParentRow.VRPECURVAB;
            wCurrentRow.VRPECURVAC = wParentRow.VRPECURVAC;
        }, 1000);
    };

    /* **************************************************
     * Função que checa a parametrização das curvas A, B
     * e C. Exibe mensagens de erro caso alguma das
     * curvas esteja nula ou a soma total de percentuais
     * ultrapasse 100%.
     *
     * @param Widget widget
     *
     * @return boolean
     */
    self.checkCurves = function(widget) {
        var tot = 0;
        //checks A curve validity
        if (!widget.currentRow.VRPECURVAA || widget.currentRow.VRPECURVAA <= 0) {
            ScreenService.showMessage("Valor do percentual da curva A deve ser maior que zero.");

            return false;
        }
        else //adds valid A curve value to total
            tot += parseFloat(widget.currentRow.VRPECURVAA);

        //checks B curve validity
        if (!widget.currentRow.VRPECURVAB || widget.currentRow.VRPECURVAB <= 0) {
            ScreenService.showMessage("Valor do percentual da curva B deve ser maior que zero.");

            return false;
        }
        else //adds valid B curve value to total
            tot += parseFloat(widget.currentRow.VRPECURVAB);

        //checks C curve validity
        if (!widget.currentRow.VRPECURVAC || widget.currentRow.VRPECURVAC <= 0) {
            ScreenService.showMessage("Valor do percentual da curva C deve ser maior que zero.");

            return false;
        }
        else //adds valid C curve value to total
            tot += parseFloat(widget.currentRow.VRPECURVAC);

        //checks total percentage validity
        if (tot != 100) {
            ScreenService.showMessage("O somatório dos percentuais da curva ABC deve totalizar 100%");

            return false;
        }
        else
            return true;
    };

    /* **************************************************
     * Procedimento que checa a disponibilidade do campo
     * de realização diária de inventário rotativo
     * de acordo com a parametrização de estoque.
     *
     * @param Widget widget
     */
    self.checkDailyRealization = function(widget) {
        //checks relevant stock parametrization
        var check = !((widget.currentRow.IDINVESTCEGO == 'N') &&
            ((widget.currentRow.IDINVROTEST == 'S') ||
                (widget.currentRow.IDINVEROTREQ == 'S') ||
                (widget.currentRow.VRPERCINVROT > 0)));

        var subField = widget.getField('IDINVROTDIA');

        //changes field avaliability according to parametrization
        subField.readOnly = check;

        if (check) //in case of field unavailability, sets negative parametrization
            widget.currentRow.IDINVROTDIA = 'N';

        //checks rotative inventory days widget
        self.checkRealizationDaysWidget(widget);
    };

    /* **************************************************
     * Procedimento que checa a validade do campo de dia
     * para fechamento do mês e altera a disponibilidade
     * do campo de realização de cálculo da receita bruta
     * de acordo.
     *
     * @param Widget widget
     */
    self.checkMonthClosureDay = function(widget) {
        var day = widget.currentRow.NRDIAFECHMES;
        var subField = widget.getField('IDCALCRECBRUTA');

        //in case of invalid date
        if (day !== null && (1 > day || day > 31)) {
            //cleans invalid data
            widget.currentRow.NRDIAFECHMES = null;
            ScreenService.showMessage("Dia para fechamento do mês deve estar entre 1 e 31.");
        }

        //check = true if date is invalid
        var check = !(day >= 1 && day <= 31);

        //changes field avaliability according to parametrization
        subField.readOnly = check;

        if (check) { //in case of field unavailability, sets negative parametrization
            widget.currentRow.IDCALCRECBRUTA = 'N';
            widget.getField('IDINVROTDIA').readOnly = false;
        }
        else {
            widget.getField('IDINVROTDIA').readOnly = true;
        }

    };

    /* **************************************************
     * Procedimento que checa a validade do campo de
     * calendário de dias não-úteis e altera a
     * disponibilidade dos campos do field group de dias
     * não-úteis de acordo.
     *
     * @param Widget widget
     */
    self.checkNonBusinessDayCalendarConsideration = function(widget) {
        var check = widget.currentRow.IDCONSDIAUTFEC != 'S';

        //checks every field in the widget
        widget.fields.forEach(function(field) {
            //checks every non business day field
            if (field.fieldGroup == 'DNU') {
                field.readOnly = check;

                if (check) //in case of field unavailability, sets negative parametrization
                    widget.currentRow[field.name] = 'N';
            }
        });
    };

    /* **************************************************
     * Função que checa a validade da parametrização de
     * inventário rotativo, quanto aos dias de realização
     * de inventário.
     *
     * @param Widget widget
     */
    self.checkRealizationDays = function(widget) {
        //checks if the widget is enabled and if there's no saved data in it
        if (widget.enabled && widget.dataSource.data && widget.dataSource.data.length < 1) {
            //activates invalid widget to guide user
            if (!widget.isActive())
                widget.activate();

            ScreenService.showMessage("Dia para inventário deve ser informado.");

            return false;
        }
        else //valid widget
            return true;
    };

    /* **************************************************
     * Procedimento que controla a visibilidade das
     * actions da aba de Dias do Inventário Rotativo de
     * acordo com o estado atual (visualização, inserção)
     *
     * @param Widget widget
     */
    self.checkRealizationDaysActionsVisibility = function(widget) {
        //checks if parent widget is being edited
        var status = widget.enabled && widget.getParent().widgets[0].mode === 'edit';
        var actions = ["BACK_LIST", "BACK_EDIT", "ADD", "DELETE", "SAVE"];

        //switches action visibility accordingly
        actions.forEach(function(actionName) {
            widget.getAction(actionName).isVisible = status;
        });
    };

    /* **************************************************
     * Procedimento que checa a disponibilidade da aba de
     * dias do inventário rotativo de acordo com a
     * parametrização de estoque.
     *
     * CAUTION! Esse procedimento DELETA todos os dias de
     * realização de inventário caso a widget esteja
     * desabilitada!
     *
     * @param Widget widget
     */
    self.checkRealizationDaysWidget = function(formWidget) {
        var dailyField = formWidget.getField('IDINVROTDIA');
        var gridWidget = formWidget.container.getWidget('diasrotativo');

        //checks daily field to avoid repeating extensive validation calculations
        if (dailyField.value() == 'S' || dailyField.readOnly)
            //removes all saved days in case of widget
            est53000_parametros_de_estoque_service.deleteAllRealizationDays(formWidget.currentRow).then(function() {
                //disables tab
                //gridWidget.enabled = false;
            });
        else {
            //enables tab
            gridWidget.enabled = true;

            //constructs datasource filter
            gridWidget.dataSourceFilter = [{
                "name": "CDFILIAL",
                "operator": "=",
                "value": formWidget.currentRow.CDFILIAL
            }];
        }

        //checks grid widget actions
        self.checkRealizationDaysActionsVisibility(gridWidget);
    };

    /* **************************************************
     * Procedimento que checa a parametrização de
     * bloqueio de lançamento e modifica a
     * disponibilidade do campo de dia de bloqueio de
     * acordo.
     *
     * @param Widget widget
     */
    self.checkReleaseBlockingForClosedMonths = function(widget) {
        var check = widget.currentRow.IDBLQESTFMES != 'S';
        var numberField = widget.getField('NRDIABLOQEST');

        //changes field avaliability according to parametrization
        numberField.readOnly = check;

        //verifies if current condition is diferent from what was computed and executes changes as necessary
        if (!check ^ numberField.validations.required)
            this.switchFieldRequirement(numberField);

        if (check) { //in case of negative parametrization, cleans field data
            delete numberField.validations.min;
            delete numberField.validations.max;
            widget.currentRow.NRDIABLOQEST = null;
        }
        else {
            numberField.validations.min = {};
            numberField.validations.max = {};
            numberField.validations.min.limit = 1;
            numberField.validations.max.value = 31;
        }
    };

    /* **************************************************
     * Procedimento que checa as parametrizações de
     * realização de inventário rotativo. Bloqueia uma
     * opção se a outra estiver validamente marcada.
     *
     * Exibe mensagem de erro no caso da marcação
     * inválida da realização por requisição de
     * alimentação escolar.
     *
     * @param Widget widget
     */
    self.checkRotativeInventory = function(widget) {
        //checks stock position inventory realization option
        if (widget.currentRow.IDINVROTEST == 'S') {
            //blocks the other inventory realization option
            widget.currentRow.IDINVEROTREQ = 'N';

            widget.getField('IDINVEROTREQ').readOnly = true;
        }
        else //unblocks the other inventory realization option
            widget.getField('IDINVEROTREQ').readOnly = false;

        //checks school feeding requisition inventory realization option
        if (widget.currentRow.IDINVEROTREQ == 'S') {
            //checks school feeding requisition validity
            if (!widget.getParent().currentRow.CDFILCENTPME) {
                //blocks operation if it is invalid
                ScreenService.showMessage("Operação bloqueada! Unidade não associada a uma central merenda.");
                widget.currentRow.IDINVEROTREQ = 'N';
                //unblocks the other inventory realization option
                widget.getField('IDINVROTEST').readOnly = false;
            }
            else {
                //in case of validity, blocks the other inventory realization option
                widget.currentRow.IDINVROTEST = 'N';

                widget.getField('IDINVROTEST').readOnly = true;
            }
        }
        else //unblocks the other inventory realization option
            widget.getField('IDINVROTEST').readOnly = false;

        //checks whether one of the inventory realization options is enabled
        var check = widget.currentRow.IDINVROTEST == 'S' || widget.currentRow.IDINVEROTREQ == 'S';
        var subFieldNames = ["VRPERCINVROT", "IDINVROTFMES"];

        //iterates over every dependent field
        subFieldNames.forEach(function(fieldName) {
            var field = widget.getField(fieldName);

            //changes field avaliability according to parametrization
            field.readOnly = check;

            if (check) //in case of negative parametrization, cleans field data
                widget.currentRow[fieldName] = null;
        });
    };

    /* **************************************************
     * Procedimento que checa a disponibilidade do campo
     * "Tipo de Inventário Rotativo no Fechamento Mês" de
     * acordo com a parametrização de estoque.
     *
     * @param Widget widget
     */
    self.checkRotativeInventoryOnMonthlyClosing = function(widget) {
        //checks relevant stock parametrization
        var check = !((widget.currentRow.IDINVROTEST == 'N') &&
            (widget.currentRow.IDINVEROTREQ == 'N') &&
            (widget.currentRow.IDINVESTCEGO == 'N') &&
            (widget.currentRow.VRPERCINVROT > 0));

        var subField = widget.getField('IDINVROTFMES');

        //changes field avaliability according to parametrization
        subField.readOnly = check;

        if (check) //in case of field unavailability, sets negative parametrization
            widget.currentRow.IDINVROTFMES = 'S';
    };

    /* **************************************************
     * Procedimento que inicia o procedimento de edição
     * de uma widget à partir de seu nome.
     *
     * @param Widget widget
     */
    self.editWidget = function(widget) {
        //searches for widget editing code block
        switch (widget.name) {
            case 'paramsrotativo':
                var tabbedWdgt = widget.getParent();
                var formWidget = tabbedWdgt.widgets[0];
                var gridWidget = tabbedWdgt.widgets[1];

                //starts editing process and revokes access to sibling and superior widgets
                General.widgetEdit(formWidget);
                let widgetsFilhas = tabbedWdgt.getParent().widgets.concat(tabbedWdgt.getParent().getParent().widgets);
                widgetsFilhas = widgetsFilhas.filter(widgets => widget.name !== 'diasrotativo');

                General.changeWidgetAcessibility(widgetsFilhas, false);

                //checks grid widget actions
                self.checkRealizationDaysActionsVisibility(gridWidget);
                gridWidget.enabled = true;

                //creates grid widget data backup
                self.backupRealizationDaysData(gridWidget);

                break;
            case 'diasrotativo':
                let widgetParams = widget.container.getWidget('paramsrotativo');
                let widgetInventario = widget.container.getWidget('inventariorotativo');
                let otherWidgets = widgetInventario.getParent().widgets.concat(widgetInventario.getParent().getParent().widgets);
                otherWidgets = otherWidgets.filter(widgets => widget.name !== 'diasrotativo');

                General.changeWidgetAcessibility(otherWidgets, false);
                let widgetDias = widget.container.getWidget('diasrotativo');
                //checks grid widget actions
                widgetDias.mode = 'edit';
                var ignoredKeys = ["__rightActions", "__leftActions", "_children", "_parent", "__groupHeaders", "__index", "__isSelected", "__notSelectable", "$$hashKey"];
                //faz backup da currentRow para permitir a recuperação dos dados
                widgetParams.backupRow = Util.clone(widgetParams.currentRow, ignoredKeys);
                self.checkRealizationDaysActionsVisibility(widgetDias);

                //creates grid widget data backup
                self.backupRealizationDaysData(widgetDias);
                break;
        }
    };

    /* **************************************************
     * Procedimento que solicita os dados de
     * parametrização de estoque geral de acordo com a
     * organização logada e replica o retorno para todas
     * as subwidgets da aba "Geral".
     *
     * @param Widget widget
     */
    self.getDataForGeralWidget = function(widget) {
        //requests data for "Geral" widget
        est53000_parametros_de_estoque_service.getDataForGeralWidget().then(function(data) {
            //sets data on "Geral" widget
            widget.currentRow = data;
            var widgetLancamento = widget.container.getWidget('lancamentos');

            //replicates data for all "Geral" tab widgets
            self.replicateDataForChildren(widget);

            if (!data.NRMESANTEMOVIEST) {
                var fieldInterval = widgetLancamento.getField('DTINTERVMOVIEST');
                fieldInterval.isVisible = true;
                widgetLancamento.getField('NRMESANTEMOVIEST').isVisible = false;
                widgetLancamento.getField('TIPOGERACAO').value('DEATE');
                fieldInterval.start = data.DTMOVIESTINI;
                fieldInterval.end = data.DTMOVIESTFIN;
                if (!widgetLancamento.currentRow.DTINTERVMOVIEST) {
                    widgetLancamento.currentRow.DTINTERVMOVIEST = {};
                    widgetLancamento.currentRow.DTINTERVMOVIEST.start = data.DTMOVIESTINI;
                    widgetLancamento.currentRow.DTINTERVMOVIEST.end = data.DTMOVIESTFIN;
                }
            }
            else {
                widgetLancamento.getField('DTINTERVMOVIEST').isVisible = false;
                widgetLancamento.getField('NRMESANTEMOVIEST').isVisible = true;
                widgetLancamento.getField('TIPOGERACAO').value('MES');
                if (!widgetLancamento.currentRow.DTINTERVMOVIEST) {
                    widgetLancamento.currentRow.DTINTERVMOVIEST = {};
                    widgetLancamento.currentRow.DTINTERVMOVIEST.start = data.DTMOVIESTINI;
                    widgetLancamento.currentRow.DTINTERVMOVIEST.end = data.DTMOVIESTFIN;
                }
            }

        }, function(error) {
            if (error) {
                var errorMessage = 'Timeout'; //default value

                //if theres an appropriate error message, gets it
                if (error.data)
                    errorMessage = error.data.error;

                //shows error message
                ScreenService.showMessage(errorMessage);
            }
        });
    };

    /* **************************************************
     * Procedimento que solicita os dados do field de
     * produtos da tab de mesmo nome dentro de CurvaABC.
     *
     * @param Field field
     */
    self.getDataForProdutoField = function(field) {
        //requests data for "Produto" field
        if (field.dataSource.data.length == 0) {
            est53000_parametros_de_estoque_service.getDataForProdutoField().then(function(data) {
                //sets data on "Produto" field
                field.dataSource.data = data;
            }, function(error) {
                if (error) {
                    var errorMessage = 'Timeout'; //default value

                    //if theres an appropriate error message, gets it
                    if (error.data)
                        errorMessage = error.data.error;

                    //shows error message
                    ScreenService.showMessage(errorMessage);
                }
            });
        }
    };


    /* **************************************************
     * Procedimento que solicita os dados da tab Produto
     * de CurvaABC.
     *
     * @param Widget widget
     */
    self.getDataForProdutoWidget = function(widget) {
        //requests data for "Produto" widget
        est53000_parametros_de_estoque_service.getDataForProdutoWidget(widget.currentRow).then(function(data) {
            //sets data on "Produto" widget
            var gridProduto = templateManager.container.getWidget('produtosgrid');
            gridProduto.dataSource.data = data;
            self.controlaActionAddProduto(widget);


        }, function(error) {
            if (error) {
                var errorMessage = 'Timeout'; //default value

                //if theres an appropriate error message, gets it
                if (error.data)
                    errorMessage = error.data.error;

                //shows error message
                ScreenService.showMessage(errorMessage);
            }
        });
    };

    self.controlaActionAddProduto = function(widget) {
        var gridProdutoFiltro = templateManager.container.getWidget('produtos');
        if (gridProdutoFiltro.currentRow.IDUTILALMOX == 'S') {
            if (gridProdutoFiltro.currentRow.IDUTILLCESTQ == 'S') {
                if (gridProdutoFiltro.currentRow.CDLOCALESTOQ) {
                    self.checkCurvaABCsProductsGrid(widget, true);
                }
                else {
                    self.checkCurvaABCsProductsGrid(widget, false);
                }
            }
            else {
                if (gridProdutoFiltro.currentRow.CDALMOXARIFE) {
                    self.checkCurvaABCsProductsGrid(widget, true);
                }
                else {
                    self.checkCurvaABCsProductsGrid(widget, false);
                }
            }
        }
        else {
            self.checkCurvaABCsProductsGrid(widget, true);
        }
    };

    /* **************************************************
     * Procedimento que solicita os dados de
     * parametrização de estoque de uma unidade e replica
     * o retorno para toda a árvore de subwidgets da aba
     * "Unidade", ignorando aquelas do tipo "grid".
     *
     * @param Widget widget
     */
    self.getDataForUnidadeWidget = function(widget) {
        //requests data for "Unidade" widget
        est53000_parametros_de_estoque_service.getDataForUnidadeWidget(widget.currentRow).then(function(data) {
            //sets data on "Unidade" widget
            widget.currentRow = data;

            widget.promise.resolve();

            //replicates data for all "Unidade" tab widgets
            self.replicateDataForChildren(widget, false);
            let widgetControleEstoque = templateManager.container.getWidget('controle');
            self.setOldControlUsageOnRow(widgetControleEstoque);
        }, function(error) {
            if (error) {
                var errorMessage = 'Timeout'; //default value

                //if theres an appropriate error message, gets it
                if (error.data)
                    errorMessage = error.data.error;

                //shows error message
                ScreenService.showMessage(errorMessage);
            }
        });
    };

    /* **************************************************
     * Procedimento que replica os dados de uma widget
     * para todas a sua árvore de subwidgets, permitindo
     * que widgets do tipo "grid" sejam ignoradas.
     *
     * @param Widget  widget
     * @param boolean dontIgnoreGrids
     */
    self.replicateDataForChildren = function(widget, dontIgnoreGrids) {
        //checks parameter passage and sets default value
        if (typeof dontIgnoreGrids === 'undefined')
            dontIgnoreGrids = true;

        //checks children widget existence
        if (widget.widgets.length)
            //iterates over children widget array
            widget.widgets.forEach(function(wdg) {
                //checks if widget is to receive data
                if (dontIgnoreGrids || wdg.template.split('/').reverse()[0] != "grid.html") {
                    //cleans child widget data
                    wdg.currentRow = {};

                    //checks whether there's data on mother widget's current row
                    if (Object.keys(widget.currentRow).length)
                        //copies every column to child widget
                        Object.keys(widget.currentRow).forEach(function(key) {
                            wdg.currentRow[key] = widget.currentRow[key];
                        });

                    //checks if child widget has children and replicates data for it, keeping the same parametrization
                    if (wdg.widgets.length)
                        self.replicateDataForChildren(wdg, dontIgnoreGrids);
                }
            });
    };

    /* **************************************************
     * Função que restaura os dados da aba de Dias do
     * Inventário Rotativo de acordo com o backup gerado
     * pela função "backupRealizationDaysData".
     *
     * CAUTION! Esse procedimento deleta todos os dias
     * atualmente existentes no banco e os substitui por
     * aqueles encontrados no backup!
     *
     * @param Widget widget
     *
     * @return Promise
     */
    self.restoreRealizationDaysBackup = function(gridWidget) {
        var formWidget = gridWidget.getParent().widgets[0];

        //removes every saved day in order to have a clean environment to apply backup
        return est53000_parametros_de_estoque_service.deleteAllRealizationDays(formWidget.currentRow).then(function() {
            //checks whether there is a valid backup
            if (gridWidget.backupDataset) {
                //restores backup
                gridWidget.dataSource.data = gridWidget.backupDataset;

                //selects first row, as the saving method requires a currentRow to exist
                gridWidget.moveToFirst();
                //saves data
                gridWidget.saveRow();
            }
            else //cleans data
                gridWidget.dataSource.data = [];
        });
    };

    //procedimento de salvar os dados da tabbed unidade
    self.saveTabbedUnidade = function(widget) {
        if (widget.isValid()) {
            widget.currentRow.DSPARFILFIS = est53000_parametros_de_estoque_service.concatenaCampoParamfilial(widget.currentRow);
            ScreenService.showLoader();
            est53000_parametros_de_estoque_service.saveUnidade(widget.currentRow).then(
                function() {
                    self.reloadUnidade();
                    General.widgetCancelForm(widget);
                    General.changeWidgetAcessibility(widget.getParent().getParent().widgets, true);
                    self.controlaStatusFiltro(false);
                    ScreenService.hideLoader();
                },
                function(callback) {
                    if (callback.data) {
                        ScreenService.showMessage(callback.data.error);
                    }
                    ScreenService.hideLoader();
                }
            );
        }
    };

    /* **************************************************
     * Procedimento que executa o processo de salvamento
     * de uma widget à partir de seu nome.
     *
     * @param Widget widget
     */
    self.saveWidget = function(widget) {
        //searches for widget saving code block
        switch (widget.name) {

            case 'inventario':
                widget.currentRow.WIDGETNAME = widget.name;
                /* falls through */
            case 'lancamentos':
                /* falls through */
            case 'grupoDeOperadores':
                /* falls through */
            case 'restricoes':
                if (widget.isValid()) {
                    ScreenService.showLoader();
                    est53000_parametros_de_estoque_service.saveGeral(widget.currentRow).then(
                        function() {
                            self.reloadGeral();
                            General.widgetCancelForm(widget);
                            General.changeWidgetAcessibility(widget.getParent().getParent().widgets, true);
                            ScreenService.hideLoader();
                        },
                        function(callback) {
                            if (callback.data) {
                                ScreenService.showMessage(callback.data.error);
                            }
                            ScreenService.hideLoader();
                        }
                    );
                }
                break;
            case 'paramsrotativo':
                if (widget.isValid()) {
                    if (widget.currentRow.IDINVROTEST == "S" && widget.currentRow.IDINVEROTREQ == "S") {
                        ScreenService.showMessage("Operação Bloqueada! Filial não associada a uma central merenda.");
                        widget.mode = 'edit';
                    }
                    else {
                        let widgetDias = templateManager.container.getWidget('diasrotativo');
                        if (widget.currentRow.IDINVROTEST == "S" && widgetDias.dataSource.data.length == 0) {
                            ScreenService.showMessage("Dia de Inventário deve ser informado");
                            widget.mode = 'edit';
                        }
                        else {
                            widget.currentRow.DSPARFILFIS = est53000_parametros_de_estoque_service.concatenaCampoParamfilial(widget.currentRow);
                            ScreenService.showLoader();
                            self.checkRotativeInventory(widget);

                            est53000_parametros_de_estoque_service.saveUnidade(widget.currentRow).then(
                                function() {
                                    self.reloadUnidade();
                                    widget.template = widget.templateToView;
                                    widget.view();
                                    General.changeWidgetAcessibility(widget.getParent().getParent().widgets, true);
                                    General.changeWidgetAcessibility(widget.getParent().getParent().getParent().widgets, true);
                                    self.controlaStatusFiltro(false);
                                    ScreenService.hideLoader();
                                },
                                function(callback) {
                                    if (callback.data) {
                                        ScreenService.showMessage(callback.data.error);
                                    }
                                    ScreenService.hideLoader();
                                }
                            );
                        }
                    }
                }
                break;

            case 'fechamento':
                /* falls through */
            case 'processamento':
                /* falls through */
            case 'atualizacao':
                /* falls through */
            case 'controle':
                /* falls through */
            case 'restlanc':
                self.saveTabbedUnidade(widget);
                break;

                //saves widget "Parâmetros" under tab "CurvaABC"
            case 'parametroscurva':
                //checks validity of each curve before saving
                if (self.checkCurves(widget)) {
                    if (widget.isValid()) {
                        ScreenService.showLoader();
                        est53000_parametros_de_estoque_service.saveParametrosCurvaAbc(widget.currentRow).then(
                            function() {
                                self.reloadUnidade();
                                General.widgetCancelForm(widget);
                                General.changeWidgetAcessibility(widget.getParent().getParent().widgets.concat(widget.getParent().getParent().getParent().widgets), true);
                                self.controlaStatusFiltro(false);
                                ScreenService.hideLoader();
                            },
                            function(callback) {
                                if (callback.data) {
                                    ScreenService.showMessage(callback.data.error);
                                }
                                ScreenService.hideLoader();
                            }
                        );
                    }
                }
                break;
                //saves widget "Dias" under tab "Inventário Rotativo"
            case 'diasrotativo':
                //checks widget mode to assert current status
                //if it is in 'edit' mode, it is trying to insert a day to the grid
                //if it is in 'list' mode, it is trying to conclude rotative inventory's parametrization
                if (widget.mode === 'edit') {
                    //checks day validity
                    if (widget.currentRow.NRDIAINVFILI < 1 || widget.currentRow.NRDIAINVFILI > 31)
                        ScreenService.showMessage("Dia para realização de inventário deve estar entre 1 e 31.");
                    else { //saves data
                        widget.currentRow.CDFILIAL = widget.getParent().currentRow.CDFILIAL;
                        General.widgetSave(widget);
                    }

                    //ends saving process
                    break;
                    /* falls through */ //it'll reach this point only if the widget is in 'list' mode
                }
                else if (widget.mode === 'list') {
                    let widgetParamsRot = templateManager.container.getWidget('paramsrotativo');
                    widgetParamsRot.activate();
                    self.saveWidget(widgetParamsRot);
                    break;
                }
                break;

            case 'produtosgrid':
                widget.currentRow.CDFILIAL = widget.getParent().currentRow.CDFILIAL;
                widget.currentRow.CDALMOXARIFE = widget.getParent().currentRow.CDALMOXARIFE;
                widget.currentRow.CDLOCALESTOQ = widget.getParent().currentRow.CDLOCALESTOQ;
                if (widget.isValid()) {
                    if (widget.currentRow.__is_new) {
                        ScreenService.showLoader();
                        est53000_parametros_de_estoque_service.validateProduct(widget.currentRow).then(function(isValid) {
                            if (isValid) {
                                self.saveProduct(widget);
                            }
                            ScreenService.hideLoader();
                        }, function(error) {
                            if (error) {
                                var errorMessage = 'Timeout'; //default value

                                //if theres an appropriate error message, gets it
                                if (error.data)
                                    errorMessage = error.data.error;

                                //shows error message
                                ScreenService.showMessage(errorMessage);
                            }
                            ScreenService.hideLoader();
                        });
                    }
                    else {
                        widget.currentRow.__is_new = false;
                        self.saveProduct(widget);
                    }
                }
                break;


                //saves widget "Inventário Rotativo"
            case 'inventariorotativo':
                //saves data
                self.saveRotativeInventory(widget);
                break;
        }
    };

    self.saveProduct = function(widget) {
        est53000_parametros_de_estoque_service.saveProduct(widget.currentRow).then(
            function() {
                if (widget.currentRow.__is_new) {
                    ScreenService.closeSwipe().then(function() {
                        self.reloadProduto(widget);
                        widget.list();
                    });
                }
                else {
                    widget.view();
                }
            },
            function(callback) {
                if (callback.data) {
                    ScreenService.showMessage(callback.data.error);
                }
            }
        );
    };

    /* **************************************************
     * Procedimento que lida com um defeito dito
     * "incorrigível" do framework, que faz o modo padrão
     * de abertura de uma widget do tipo form ser "list"
     * ao invés de "view".
     *
     * @param Widget widget
     */
    self.setForms = function(widget) {
        //sets form widget to view mode if it is in list mode
        if (widget.mode === 'list')
            widget.view();
    };

    ////////////////////////////////////////////////////////
    // -------------------------------------------------- //
    // -----------[UNDOCUMENTED/WIP FUNCTIONS]----------- //
    // -------------------------------------------------- //
    ////////////////////////////////////////////////////////

    /* **************************************************
     * Procedimento que efetua troca de tabs pra evitar erro de grid 
     * no inicio da pagina
     *
     */
    self.openCloseTab = function() {
        let widgetGeral = templateManager.container.getWidget('geral');
        let widgetLancamentos = templateManager.container.getWidget('lancamentos');
        let widgetUnidade = templateManager.container.getWidget('unidade');
        let widgetTabbed = templateManager.container.getWidget('tabbed');

        ScreenService.showLoader();

        setTimeout(() => {
            widgetUnidade.enabled = true;
            widgetTabbed.openTab("unidade");
            setTimeout(() => {
                widgetGeral.enabled = true;
                widgetTabbed.openTab("geral");
                setTimeout(() => {
                    widgetLancamentos.activate();
                    setTimeout(() => {
                        if (!widgetLancamentos.isActive())
                            widgetLancamentos.activate();
                        setTimeout(() => {
                            if (!widgetLancamentos.isActive()) {
                                self.openCloseTab();
                            }
                            else {
                                ScreenService.hideLoader();
                            }
                        }, 500);
                    }, 500);
                }, 1000);
            }, 2000);
        }, 200);
    };

    /* **************************************************
     * Procedimento que efetua uma checagem padrão de uma
     * widget à partir de seu nome.
     *
     * @param Widget widget
     */

    self.checkWidget = function(widget) {
        //searches for widget validation code block
        switch (widget.name) {
            case 'moedaForteTabbed':
                this.checkAutofillFields(widget);
                break;
                //checks widget "Atualização de Estoque"
            case 'atualizacao':
                self.checkReleaseBlockingForClosedMonths(widget);
                break;
                //checks widget "Controle de Estoque"
            case 'controle':
                self.checkControlUsage(widget);
                self.checkAllotment(widget);
                self.checkAllotmentNumberDescription(widget);
                break;
                //checks widget "Fechamento do Dia"
            case 'fechamento':
                self.checkNonBusinessDayCalendarConsideration(widget);
                break;
                //checks widget "Geral"
            case 'geral':
                self.getDataForGeralWidget(widget);
                break;
                //checks widget "Parâmetros" under "Inventário Rotativo"
            case 'paramsrotativo':
                self.checkBranchForServiceForStockReductionField(widget);
                self.checkRotativeInventory(widget);
                self.checkRotativeInventoryOnMonthlyClosing(widget);
                self.checkDailyRealization(widget);
                self.checkMonthClosureDay(widget);
                self.checkBlindInventoryRealization(widget);
                self.checkAutomaticInventoryDeploymentGeneration(widget);
                break;
                //checks widget "Produtos"
            case 'produtos':
                widget.currentRow.CDALMOXARIFE = null;
                widget.currentRow.DSALMOXARIFE = null;

                self.checkCurvaABCsProductsFieldsFilters(widget);
                self.checkCurvaABCsProductsWarehouse(widget);
                break;
                //checks widget "Unidade"
            case 'unidade':

                self.checkBranch(widget);
                break;
        }
    };

    this.setCurrentRowUnidadeLogada = function(widget) {
        this.getFilialLogadaOperador().then(function(success) {
            let filial = success.dataset.filial[0];

            widget.currentRow.CDFILIAL = filial.CDFILIAL;
            widget.currentRow.NMFILIAL = filial.NMFILIAL;

            self.replicateDataForChildren(widget);

        }.bind(this));
    };

    this.getFilialLogadaOperador = function() {
        var filter = [{
            "name": "CDFILIAL",
            "operator": "=",
            "value": "FILIAL_LOGADA"
        }];
        return General.callRestEngine('est#/crud/filial', 'FilterData', filter);
    };

    self.checkCurvaABCsProductsFieldsFilters = function(tabbedWdgt) {
        var CDFILIAL = tabbedWdgt.currentRow.CDFILIAL;

        tabbedWdgt.getField('CDALMOXARIFE').dataSourceFilter[0].value = CDFILIAL;
        tabbedWdgt.getField('CDLOCALESTOQ').dataSourceFilter[0].value = CDFILIAL;

        tabbedWdgt.getField('CDALMOXARIFE').reload();
        tabbedWdgt.getField('CDLOCALESTOQ').reload();

    };

    self.checkCurvaABCsProductsWarehouse = function(tabbedWdgt) {
        var check = tabbedWdgt.currentRow.IDUTILALMOX == 'S';
        var field = tabbedWdgt.getField('CDALMOXARIFE');
        var subField = tabbedWdgt.getField('CDLOCALESTOQ');

        if (check) {
            field.readOnly = false;
            check = !!(tabbedWdgt.currentRow.CDALMOXARIFE);

            if (check) {
                subField.readOnly = false;

                check = tabbedWdgt.currentRow.IDUTILLCESTQ == 'S';

                if (check) {
                    if (tabbedWdgt.currentRow.CDALMOXARIFE != subField.dataSourceFilter[1].value) {
                        subField.dataSourceFilter[1].value = tabbedWdgt.currentRow.CDALMOXARIFE;

                        tabbedWdgt.currentRow.CDLOCALESTOQ = null;
                        tabbedWdgt.currentRow.DSLOCALESTOQ = null;

                        subField.reload();

                        self.clearCurvaABCsProductsGrid(tabbedWdgt);
                    }
                    else
                        self.checkCurvaABCsProducts(tabbedWdgt);
                }
                else {
                    subField.readOnly = true;

                    tabbedWdgt.currentRow.CDLOCALESTOQ = ' ';
                    tabbedWdgt.currentRow.DSLOCALESTOQ = ' ';

                    self.checkCurvaABCsProducts(tabbedWdgt);
                }
            }
            else {
                subField.readOnly = true;

                tabbedWdgt.currentRow.CDLOCALESTOQ = null;
                tabbedWdgt.currentRow.DSLOCALESTOQ = null;

                self.clearCurvaABCsProductsGrid(tabbedWdgt);
            }
        }
        else {
            field.readOnly = true;

            subField.readOnly = true;
            tabbedWdgt.widgets[0].dataSource.data = null;
            tabbedWdgt.widgets[0].getAction('ADD').readOnly = false;

            tabbedWdgt.currentRow.CDALMOXARIFE = ' ';
            tabbedWdgt.currentRow.DSALMOXARIFE = ' ';

            tabbedWdgt.currentRow.CDLOCALESTOQ = ' ';
            tabbedWdgt.currentRow.DSLOCALESTOQ = ' ';

            var gridWidget = tabbedWdgt.widgets[0];

            gridWidget.currentRow.CDFILIAL = gridWidget.getParent().currentRow.CDFILIAL;
            gridWidget.currentRow.CDALMOXARIFE = gridWidget.getParent().currentRow.CDALMOXARIFE;
            gridWidget.currentRow.CDLOCALESTOQ = gridWidget.getParent().currentRow.CDLOCALESTOQ;

            self.getDataForProdutoWidget(gridWidget);

        }
    };

    self.checkCurvaABCsProducts = function(tabbedWdgt) {
        var gridWidget = tabbedWdgt.widgets[0];
        var check = (tabbedWdgt.currentRow.IDUTILALMOX == 'S' &&
                tabbedWdgt.currentRow.CDALMOXARIFE) &&
            (tabbedWdgt.currentRow.IDUTILLCESTQ == 'S' ||
                tabbedWdgt.currentRow.CDLOCALESTOQ);

        if (check) {
            gridWidget.currentRow.CDFILIAL = gridWidget.getParent().currentRow.CDFILIAL;
            gridWidget.currentRow.CDALMOXARIFE = gridWidget.getParent().currentRow.CDALMOXARIFE;
            gridWidget.currentRow.CDLOCALESTOQ = gridWidget.getParent().currentRow.CDLOCALESTOQ;

            self.getDataForProdutoWidget(gridWidget);
        }
    };

    self.clearCurvaABCsProductsGrid = function(tabbedWdgt) {
        var gridWidget = tabbedWdgt.widgets[0];

        gridWidget.dataSource.data = [];
        self.checkCurvaABCsProductsGrid(gridWidget, false);
    };

    self.checkCurvaABCsProductsGrid = function(widget, enabled) {
        widget.actions.forEach(function(action) {
            action.readOnly = !enabled;
        });

        widget.reload();
        templateManager.updateTemplate();
    };

    self.saveRotativeInventory = function(tabbedWdgt) {
        // var formWidget = tabbedWdgt.widgets[0];
        var gridWidget = tabbedWdgt.widgets[1];

        if (self.checkRealizationDays(gridWidget)) {
            ScreenService.showMessage("I SHOULD SAVE \"" + tabbedWdgt.label + "\"'S DATA");
            // General.widgetSave(widget);
            // self.paradaquemudaasactionsdagridwidget
            // General.changeWidgetAcessibility(widget.getParent().getParent().widgets.concat(widget.getParent().getParent().getParent().widgets), true);
        }
    };

    //recarregar dados da tabbed Geral
    self.reloadGeral = function() {
        var widget = templateManager.container.getWidget('geral');
        self.getDataForGeralWidget(widget);
    };

    //recarregar dados da tabbed Unidade
    self.reloadUnidade = function() {
        var widget = templateManager.container.getWidget('unidade');
        self.getDataForUnidadeWidget(widget);
    };

    self.reloadProduto = function() {
        var widget = templateManager.container.getWidget('produtosgrid');
        // so ira permirtir recarregar a tela caso o campo de almoxarifado esteja preenchido;
        if (widget.getParent().getField('CDALMOXARIFE').value()) {

            if (widget.getParent().getField('CDALMOXARIFE').value() !== ' ') {
                //caso a unidade utiliza localizao so irá permitir recarregar a tela caso o campo de loclaização esteja preenchido
                if (widget.getParent().currentRow.IDUTILLCESTQ == 'S') {

                    if (widget.getParent().getField('CDLOCALESTOQ').value()) {
                        if (widget.getParent().getField('CDLOCALESTOQ').value() !== ' ') {
                            self.getDataForProdutoWidget(widget.getParent());
                        }
                    }

                }
                else {
                    self.getDataForProdutoWidget(widget.getParent());
                }
            }
            else {
                self.getDataForProdutoWidget(widget.getParent());
            }
        }
    };

    //controla valicao na aba restrições nos lançamentos referentes a data
    self.verificaRestricoesDatas = function(widget, diasEntrada, diasSaida, diasAjuste, campoVerifica) {
        var diasLancoEntrada = widget.getField(diasEntrada);
        var diasLancoSaida = widget.getField(diasSaida);
        var diasLancoAjuste = widget.getField(diasAjuste);
        var verifica = widget.getField(campoVerifica);

        if (verifica.value() == 'S') {
            diasLancoSaida.readOnly = false;
            diasLancoEntrada.readOnly = false;
            diasLancoAjuste.readOnly = false;

        }
        else {
            diasLancoSaida.readOnly = true;
            diasLancoEntrada.readOnly = true;
            diasLancoAjuste.readOnly = true;
            diasLancoEntrada.value('');
            diasLancoSaida.value('');
            diasLancoAjuste.value('');
        }
    };

    self.checkValidationsDatageral = function(widget) {
        self.verificaRestricoesDatas(widget, 'NRDIASENTR', 'NRDIASSAID', 'NRDIASAJUS', 'IDLANCDTMEN');
        self.verificaRestricoesDatas(widget, 'NRDIASENTRPD', 'NRDIASSAIDPD', 'NRDIASAJUSPD', 'IDLANCDTMAI');
    };

    self.checkValidationsDataUnidade = function(widget) {
        self.verificaRestricoesDatas(widget, 'NRDIASENTRFIL', 'NRDIASSAIDFIL', 'NRDIASAJUSFIL', 'IDLANCDTMENFIL');
        self.verificaRestricoesDatas(widget, 'NRDIASENTRPDFIL', 'NRDIASSAIDPDFIL', 'NRDIASAJUSPDFIL', 'IDLANCDTMAIFIL');
    };

    self.setarValorAntigoCurvaAbc = function(widget) {
        widget.currentRow.IDCURVAABC_OLD = widget.currentRow.IDCURVAABC;
    };

    self.deleteProdutoCurvaAbc = function(widget) {
        ScreenService.confirmMessage(ScreenService.i18n('Excluir Registro?'), 'question', function() {
            est53000_parametros_de_estoque_service.deleteProdutoCurvaAbc(widget.currentRow).then(function() {
                ScreenService.closeSwipe().then(function() {
                    widget.activate();
                    self.reloadProduto();
                });
            }, function(error) {
                if (error.data && error.data.error.indexOf('ORA-02292') !== -1) {
                    ScreenService.showMessage('Exclusão não permitida! O(s) registro(s) possui(em) registros filhos.');
                }
                else {
                    ScreenService.showMessage(error.data.error);
                }
            });
        });
    };

    self.recarregarField = function(nomeWidget, nomeField) {
        var widget = templateManager.container.getWidget(nomeWidget);
        widget.getField(nomeField).reload().then(
            function() {
                if (nomeField == 'CDALMOXARIFE' || nomeField == 'CDLOCALESTOQ') {
                    self.verificaCadastroAlmoxarifadoLocalizacao(widget.getField(nomeField));
                }
            },
            function() {}
        );

    };

    // apaga valor da localização toda vez que alterar o almoxaridado
    self.controlaAlmoxarifadoLocalizacao = function(widget) {
        widget.currentRow.CDLOCALESTOQ = '';
        widget.currentRow.DSLOCALESTOQ = '';
    };

    self.verificaCadastroAlmoxarifadoLocalizacao = function(field) {
        if (field.name == 'CDALMOXARIFE') {
            if (field.dataSource.data.length === 0) {
                ScreenService.showMessage('Não há almoxaridado cadastrado(a) para a unidade informada.');
            }
        }
        else if (field.name == 'CDLOCALESTOQ') {
            if (field.dataSource.data.length === 0) {
                ScreenService.showMessage('Não há localização cadastrado(a) para o almoxaridado informado.');
            }
        }
        else if (field.name == 'CDGRUPPRCTRLTEM' || field.name == 'CDGRUPPRMOVEST') {
            if (field.dataSource.data.length === 0) {
                ScreenService.showMessage('Não há agrupamento cadastrado.');
            }
        }


    };

    // 
    self.alteraRealizaInventario = function(widget) {
        //checks stock position inventory realization option
        if (widget.currentRow.IDINVROTEST == 'S') {
            //blocks the other inventory realization option
            widget.currentRow.IDINVEROTREQ = 'N';

            widget.getField('IDINVEROTREQ').readOnly = true;
        }
        else //unblocks the other inventory realization option
            widget.getField('IDINVEROTREQ').readOnly = false;

        //checks school feeding requisition inventory realization option
        if (widget.currentRow.IDINVEROTREQ == 'S') {
            //checks school feeding requisition validity
            if (!widget.getParent().currentRow.CDFILCENTPME) {
                //blocks operation if it is invalid
                ScreenService.showMessage("Operação bloqueada! Unidade não associada a uma central merenda.");

                widget.currentRow.IDINVEROTREQ = 'N';

                //unblocks the other inventory realization option
                widget.getField('IDINVROTEST').readOnly = false;
            }
            else {
                //in case of validity, blocks the other inventory realization option
                widget.currentRow.IDINVROTEST = 'N';

                widget.getField('IDINVROTEST').readOnly = true;
            }
        }
        else //unblocks the other inventory realization option
            widget.getField('IDINVROTEST').readOnly = false;

        //checks whether one of the inventory realization options is enabled
        var check = widget.currentRow.IDINVROTEST == 'S' || widget.currentRow.IDINVEROTREQ == 'S';
        var subFieldNames = ["VRPERCINVROT", "IDINVROTFMES"];

        //iterates over every dependent field
        subFieldNames.forEach(function(fieldName) {
            var field = widget.getField(fieldName);

            //changes field avaliability according to parametrization
            field.readOnly = check;

            if (check) //in case of negative parametrization, cleans field data
                widget.currentRow[fieldName] = null;
        });
    };

    //função para não permetir que se troque o filtro de unidade enquanto se edita um registro
    self.controlaStatusFiltro = function(status) {
        var widget = templateManager.container.getWidget('unidade');
        widget.getField('NMFILIAL').readOnly = status;
    };

    self.changeMoviestParameter = function(fieldMoviestType) {
        let widgetLancamento = fieldMoviestType.widget;
        let fieldInterval = widgetLancamento.getField('DTINTERVMOVIEST');
        let fieldMonthNumber = widgetLancamento.getField('NRMESANTEMOVIEST');
        if (fieldMoviestType.value() == 'MES') {
            fieldMonthNumber.isVisible = true;
            fieldInterval.isVisible = false;
            fieldInterval.start = null;
            fieldInterval.end = null;
            widgetLancamento.currentRow.DTINTERVMOVIEST.start = null;
            widgetLancamento.currentRow.DTINTERVMOVIEST.end = null;
            templateManager.updateTemplate();

        }
        else {
            fieldMonthNumber.isVisible = false;
            fieldInterval.isVisible = true;
            fieldMonthNumber.value(null);
            templateManager.updateTemplate();
        }

    };

    this.validaHora = ({ owner: { field } }) => {
        const HORA = field.value() || false;
        let toTime = moment(HORA, 'hh:mm');
        if (toTime._d == 'Invalid Date') {
            ScreenService.showMessage('Hora Invalida');
            field.setValue('00:00');
        }

        if (toTime._d >= moment("08:00", "hh:mm") && toTime._d <= moment("18:00", "hh:mm")) {
            ScreenService.showMessage('Hora Inválida! A movimentação de estoque não deve ser agendada no período entre 08:00 às 18:00.');
            field.setValue('00:00');
        }
    };

    this.defineIdiomaTipoCustoGrid = function(widget) {
        // 	setLocalVar('language', 'es_es');
        var idioma = getLocalVar('language');
        var nomeField = null;

        if (idioma == 'pt_br')
            nomeField = 'NMTIPOCUSTO';
        else if (idioma == 'en_us')
            nomeField = 'NMTIPOCUSING';
        else if (idioma == 'es_es' || idioma == 'es_cl')
            nomeField = 'NMTIPOCUSESP';

        var fields = widget.fields;

        fields.forEach(function(field) {
            if (field.name == nomeField || field.name == 'CDTIPOCUSTO' || field.name == 'IDUTTPCUSTO' || field.name == 'INDICE')
                field.isVisible = true;
            else
                field.isVisible = false;
        });
    };

    this.defineIdiomaTipoCusto = function(widget) {
        // 	setLocalVar('language', 'en_us');
        var idioma = getLocalVar('language');
        var nomeField = null;
        var nomeFieldLabel = null;

        if (idioma == 'pt_br') {
            nomeField = 'NMTIPOCUSTO';
            nomeFieldLabel = 'NMTIPCUSTEST';
        }
        else if (idioma == 'en_us') {
            nomeField = 'NMTIPOCUSING';
            nomeFieldLabel = 'NMTIPOCUSING';
        }
        else if (idioma == 'es_es' || idioma == 'es_cl') {
            nomeField = 'NMTIPOCUSESP';
            nomeFieldLabel = 'NMTIPOCUSESP';
        }
        var fields = widget.getField('NMTIPCUSTEST').fields;

        fields.forEach(function(field) {
            if (field.name == nomeField || field.name == 'CDTIPOCUSTO')
                field.isVisible = true;
            else
                field.isVisible = false;
        });

        widget.getField('NMTIPCUSTEST').displayedFields[1] = nomeFieldLabel;
        widget.getField('NMTIPCUSTEST').searchableFields[1] = nomeFieldLabel;
        delete widget.getField('NMTIPCUSTEST').outData.NMTIPOCUSTO;
        widget.getField('NMTIPCUSTEST').outData[nomeFieldLabel] = "NMTIPCUSTEST";

        widget.currentRow.NMTIPCUSTEST = widget.getField('NMTIPCUSTEST').value();

        widget.getField('NMTIPCUSTEST').reload();

    };

    this.setTipoCusto = function(widget) {
        var widgetUnidade = templateManager.container.getWidget('unidade');
        widgetUnidade.promise.then(() => {
            var cdtipcustest = templateManager.container.getWidget('unidade').currentRow.CDTIPCUSTEST;

            if (cdtipcustest) {
                this.getTipoCusto(cdtipcustest).then(function(success) {
                    var idioma = getLocalVar('language');
                    var labelTipoCusto = null;

                    if (idioma == 'pt_br') {
                        labelTipoCusto = success.dataset.tipocusto[0].NMTIPOCUSTO;
                    }
                    else if (idioma == 'en_us') {
                        labelTipoCusto = success.dataset.tipocusto[0].NMTIPOCUSING;
                    }
                    else if (idioma == 'es_es' || idioma == 'es_cl') {
                        labelTipoCusto = success.dataset.tipocusto[0].NMTIPOCUSESP;
                    }

                    widget.currentRow.NMTIPCUSTEST = labelTipoCusto;

                });
            }

        });

    };

    this.getTipoCusto = function(cdtipocusto) {
        var filter = [
            {
                "name": "CDTIPOCUSTO",
                "operator": "=",
                "value": cdtipocusto
            }
        ];
        return General.callRestEngine('est#/crud/tipocusto', 'FilterData', filter);
    };

    this.indexHandler = function(widgetTabbed, clickedField) {
        const widgetMoedaForte = templateManager.container.getWidget('moedaForte');
        const firstIndexCode = widgetTabbed.currentRow.CDINDICEMF;
        const secondIndexCode = widgetTabbed.currentRow.CDINDICEMF2;

        this.validateDuplicatedIndex(widgetTabbed, clickedField, firstIndexCode, secondIndexCode);

        widgetTabbed.currentRow.__is_new = false;
        widgetTabbed.currentRow.CDPARGERAL = '0001';

        EstUtil.saveCrud(widgetTabbed).then(_ => {
            this.checkIndexNameOnGrid(widgetTabbed, widgetMoedaForte, firstIndexCode, secondIndexCode);
            widgetTabbed.list();
            widgetMoedaForte.activate();
            widgetTabbed.reload();
        });
    };

    this.validateDuplicatedIndex = function(widgetTabbed, clickedField, firstIndexCode, secondIndexCode) {
        if (firstIndexCode == secondIndexCode) {
            if (firstIndexCode != "" && secondIndexCode != "") {
                clickedField.setValue("");
                return ScreenService.showMessage('Índice já cadastrado!');
            }
        }
    };

    this.checkIndexNameOnGrid = function(widgetTabbed, widgetMoedaForte, firstIndexCode, secondIndexCode) {

        widgetMoedaForte.dataSource.data.forEach(item => {
            if (item.IDMOEDCUSTO == '1') {
                item.INDICE = firstIndexCode === "" ? "Índice não definido" : widgetTabbed.currentRow.NMINDICEMF;
            }
            else if (item.IDMOEDCUSTO == '2') {
                item.INDICE = secondIndexCode === "" ? "Índice não definido" : widgetTabbed.currentRow.NMINDICEMF2;
            }
            else {
                item.INDICE = "Índice não definido";
            }
        });
    };

    this.checkAutofillFields = function(widget) {
        const currentRow = widget.currentRow;
        const data = widget.dataSource.data[0];

        widget.fields.forEach(field => {
            if (field.name === 'NMINDICEMF') {
                this.getIndexData(data.CDINDICEMF).then((success) => {
                    currentRow.CDINDICEMF = data.CDINDICEMF;
                    currentRow.NMINDICEMF = success.dataset.indice.length != 0 ? success.dataset.indice[0].NMINDICE : "";
                });
            }
            else if (field.name === 'NMINDICEMF2') {
                this.getIndexData(data.CDINDICEMF2).then((success) => {
                    currentRow.CDINDICEMF2 = data.CDINDICEMF2;
                    currentRow.NMINDICEMF2 = success.dataset.indice.length != 0 ? success.dataset.indice[0].NMINDICE : "";
                });
            }
            field.reload();
        });
    };

    this.getIndexData = function(fieldValue) {
        var filter = [
            {
                "name": "CDTPINDICE",
                "operator": "=",
                "value": "1"
            },
            {
                "name": "IDPERVALOR",
                "operator": "=",
                "value": "2"
            },
            {
                "name": "CDINDICE",
                "operator": "=",
                "value": fieldValue
            }
        ];
        return General.callRestEngine('est#/crud/indice', 'FilterData', filter);
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register('Est53000_Parametros_de_Estoque_Controller', Est53000_Parametros_de_Estoque_Controller);
});
